class Blob {
  public static final float SPAWN_TIME = 2000.0;
  private PVector pos;
  float rad;
  public LinkedList bubbles;
  Circle bounding_circle;  // Circle for detecting collisions
  private int[] hit_time = new int[2]; //blob can get hit twice
  private PVector[] hit_pos = new PVector[2];
  
  Blob(int x, float r){
    pos = new PVector(x, screeny);
    rad = r;
    hit_time[0] = 0;
    hit_time[1] = 0;
    bubbles = new LinkedList();
//    bubbles.add(new Bubble(new PVector(pos.x+.8*rad, pos.y-.8*rad)));
    bounding_circle = new Circle(pos, rad);
  }
  
  //return true if blob was destroyed, otherwise false
  public boolean collide(Comet c)
  {
    int temp_hit_time = millis();

    //check each bubble for collision with the given comet
    for(int i=0; i < bubbles.size(); i++)
    {
      Bubble b = (Bubble) bubbles.get(i);

      boolean destroyed = b.collide(c);
      if(destroyed)
      {
        bubbles.remove(b);
      }
      else
      {
        Bubble new_bubble = b.regen();
        if(new_bubble != null)
        {
          bubbles.add(new_bubble);
        }
      }
    }
    if (c.mark_for_deletion) return false;
    
    //check if the comet hit the main blob
    PVector collision_pos = bounding_circle.collide(c.bcircle);
    if (collision_pos != null){ // Collision with main blob
      c.mark_for_deletion = true;
      collision_pos.sub(bounding_circle.pos);
    
      if(hit_time[0] > 0 && hit_time[1] > 0) //third hit, so die
        return true;
        
      //Play comet hit sound
      if (cometHit.position() == (cometHit.length()-1)){
        cometHit.rewind();
      }
      cometHit.play();

      int index; //0 or 1 based on where hit stuff goes
      if (hit_time[0] == 0)
        index = 0;
      else index = 1;
      hit_time[index] = temp_hit_time;
      hit_pos[index] = PVector.sub(c.bcircle.pos, pos);
    }
    
    //didn't die, so check for regeneration
    PVector child_pos = null;
    for (int i=0; i<2; i++){
      if (hit_time[i] > 0 && hit_time[i] + SPAWN_TIME < temp_hit_time){
        child_pos = hit_pos[i];
        child_pos.add(pos);
        hit_time[i] = 0;
        hit_pos[i] = null;
      }
    }
    if (child_pos != null)
    {
      //play sound
       if (regen.position() >= regen.length()){
        regen.rewind();
       }
       regen.play();
      bubbles.add(new Bubble (child_pos));
    }
    return false;
  }

  void update()
  {
    if(rightHeld && pos.x+rad/2 < screenx){
      pos.x += SPEED;
    }
    if(leftHeld && pos.x-rad/2 > 0){
      pos.x -= SPEED;
    }

    for(int i=0; i<bubbles.size(); i++){
      Bubble b = (Bubble) bubbles.get(i);
      if(leftHeld && pos.x-rad/2 > 0)
        b.moveLeft();
      if(rightHeld && pos.x+rad/2 < screenx)
        b.moveRight();
    }
  }

  void disp()
  {
    //figure out which image to use. Note there is no 'else' here.
    PImage img = blob_cleanimg;
    if(hit_time[0] + hit_time[1] > 0)
      img = blob_hurt1img; //one hole
    if(hit_time[0] + hit_time[1] > millis())
      img = blob_hurt2img; //two holes
      
//    ellipse(pos.x, pos.y, 2*rad, 2*rad);
    image(img, pos.x-rad, pos.y-rad, 2*rad, 2*rad);
    for(int i=0; i<bubbles.size(); i++){
      Bubble b = (Bubble) bubbles.get(i);
      b.disp();
    }
  }
}



