class Bubble
{
  public static final float RADIUS = 10.0;
  public static final float SPAWN_TIME = 2000.0;
  public Circle bounding_circle;
  PVector pos;

  private int hit_state;
  private int hit_time; //hit_time = 0 means it's healthy
  private PVector hit_pos;
  
  public Bubble(PVector pos)
  {
    this.bounding_circle = new Circle(pos, RADIUS);
    this.pos = pos;
    hit_time  = 0;
  }
  
  void moveRight()
  { pos.x += SPEED; }
  void moveLeft()
  { pos.x -= SPEED; }

  void disp()
  {
    if(hit_time != 0)
      image(bub_hurtimg, pos.x-RADIUS, pos.y-RADIUS, RADIUS, RADIUS);
    else image(bub_cleanimg, pos.x-RADIUS, pos.y-RADIUS, RADIUS, RADIUS);
//    if() stroke(255, 0, 0);
//    else stroke(0, 255, 0);
//    ellipse(pos.x, pos.y, RADIUS, RADIUS);
  }
  
  /**
  * Returns true if the bubble was killed, false otherwise
  */
  public boolean collide(Comet com)
  {
    Circle c = com.bcircle;
    PVector collision_pos = bounding_circle.collide(c);
    
    //No Collision
    if(collision_pos == null)
      return false;
      
    //Collision
    else
    {
      com.mark_for_deletion = true;
      //play comet hit sound
      if (cometHit.position() == (cometHit.length()-1)){
        cometHit.rewind();
      }
      cometHit.play();
      
      if (hit_time > 0)
        return true;
      
      hit_time = millis();
      hit_pos = PVector.sub(c.pos, pos);
      collision_pos.sub(bounding_circle.pos);
    }
    return false;
  }
  
  /**
  * Generates a new bubble as the child of this one and returns it
  * or returns null
  */
  public Bubble regen()
  {
    if(hit_time == 0) return null;
    if(millis() - hit_time < SPAWN_TIME) return null;

    //Grow new bubble from damaged quadrant
    hit_pos.mult(.8);
    hit_pos.add(pos);
    PVector child_pos = hit_pos;
      
    //Grow back quad
    hit_time = 0;
    hit_pos = null;
    //play sound
    if (regen.position() >= regen.length()){
      regen.rewind();
    }
    regen.play();
    return new Bubble(child_pos);
  }
}
