import processing.core.*; 
import processing.xml.*; 

import ddf.minim.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class Regen extends PApplet {



int screenx = 400;
int screeny = 500;
final int SPEED = 2;
int level = 1;
Blob blob;
LinkedList comets;
ScoreScreen score;
int counter; //counts score
boolean rightHeld = false;
boolean leftHeld = false;
int startTime = 0;  // play start time in millis
final int BLOB_X = 100;
final float BLOB_RADIUS = 20.0f;

//images from data file
PImage bg;
PImage blob_cleanimg;
PImage blob_hurt1img;
PImage blob_hurt2img;
PImage bub_cleanimg;
PImage bub_hurtimg;
PImage comet_img;
  
// Game States
final byte GS_PLAYING = 0;  // Playing game
final byte GS_HELP = 1;  // At help/learn screen
final byte GS_SCORES = 2;  // At High Score screen
final byte GS_TITLE = 3;  // At title screen
final byte GS_GAME_OVER = 4;  // At game over screen
byte gameState = GS_TITLE;

//sound
AudioSnippet bkgmusic;
AudioSnippet cometHit;
AudioSnippet regen;
AudioSnippet gameover;
AudioSnippet helpmusic;
Minim minim;

TitleScreen title;
HelpScreen  help;

public void setup() 
{
  size(screenx, screeny);
  
  loadImages();
  loadAudio();

  blob = new Blob(BLOB_X, BLOB_RADIUS);
  comets = new LinkedList();
  
  title = new TitleScreen();
  help  = new HelpScreen();
  score = new ScoreScreen();
}

public void loadAudio()
{
  minim = new Minim(this);
  bkgmusic = minim.loadSnippet("spaceloop2.wav");
  cometHit = minim.loadSnippet("comet_short.mp3");
  regen = minim.loadSnippet("effectgame.wav");
  gameover = minim.loadSnippet("comet_long.mp3");
  //actually found this annoying, but it's here if we decide to use it
  helpmusic = minim.loadSnippet("spaceloop1.mp3");
}

public void loadImages()
{
  bg = loadImage("bg.jpg");
  blob_cleanimg = loadImage("bubble6.png");
  blob_hurt1img = loadImage("bubble6_hurt.png");
  blob_hurt2img = loadImage("bubbledamagedcenter.png");
  bub_cleanimg = loadImage("extrabubble2.png");
  bub_hurtimg = loadImage("extrabubbledmg.png");
  comet_img = loadImage("cometRed.png");
}

//return the probability that a new comet is made
//at the current timestep, given the level
public float newCometProb()
{
  return (2.0f + 3.0f*level) / (60.0f + 3.0f*level);
}

//create a faller
public void makeComet()
{
  comets.add(new Comet());
}

public void die()
{
  //halt any sound effects
  bkgmusic.pause();
  cometHit.pause();
  regen.pause();
  background(bg);
  
  gameState = GS_GAME_OVER;
  gameover.play();
  text("You died :(", 160, 170);
  text("level: "+level, screenx-60, 14); //draw the level
  text("comets: "+comets.size(), 5, 14);
  text("bubbles: "+blob.bubbles.size(), 5, 28);
  text("max bubbles: "+counter, 5, 42);
  
  noLoop();
  gameover.rewind();
  // Compute score and save it if it's high
  int tmpScore = (level + 1) * (counter);
  counter = 0;
  if (score.isHighScore(tmpScore))
  {
    text("You got a high score of " + tmpScore + '!',
          150, 190);
    score.addScore(tmpScore);
    // Save score everytime there's an update,
    // in case of a sudden exit
    score.saveScores();
  }
  
  // reset everything
  startTime = 0;
  level = 0;
  blob = new Blob(BLOB_X, BLOB_RADIUS);
  comets = new LinkedList();
}

public void draw()
{
  if (bg != null)
    background(bg);
  else background(0);
  
  switch (gameState)
  {
    case (GS_TITLE):
      title.display();    // Display the title screen
      break;
      
    case (GS_HELP):       // Display the help screen
      help.display();
      break;
      
    case (GS_SCORES):     //Display the score screen
      score.display();
      break;
      
    case (GS_PLAYING):
      if (startTime == 0)     // set start to zero
        startTime = millis(); // when a new game starts
      level = (millis()-startTime)/10000;
      //sound
      if (bkgmusic.position() >= (bkgmusic.length()-1))
      {
        bkgmusic.rewind();
      }
      bkgmusic.play();
      if (random(1) < newCometProb())
        makeComet(); //create a faller
      for(int i=0; i<comets.size(); i++){
        Comet c = (Comet) comets.get(i);
        c.update();
        if(blob.collide(c)){
          die();
          return;
        }
        if(c.bcircle.pos.y > screeny || c.mark_for_deletion)
          comets.remove(c);
        stroke(0, 0, 255);
        c.disp();    
      }
      blob.update();
      blob.disp();
      
      if (blob.bubbles.size() > counter)
        counter = blob.bubbles.size();
    
      text("level: "+level, screenx-60, 14); //draw the level
      text("comets: "+comets.size(), 5, 14);
      text("bubbles: "+blob.bubbles.size(), 5, 28);
      text("max bubbles: "+counter, 5, 42);
      break;
  }  // End switch(gameState)
}

public void keyPressed()
{
  // Check which screen we're on
  switch (gameState)
  {
    case (GS_TITLE):
      if (key == ENTER || key == RETURN)
        switch (title.menuSelection)
        {
          case (TitleScreen.ID_LEARN):
            gameState = GS_HELP;
            break;
          case (TitleScreen.ID_BEGIN):
            gameState = GS_PLAYING;
            break;
          case (TitleScreen.ID_SCORES):
            gameState = GS_SCORES;
            break;
        }  // End switch(title.menuSelection)
      if (key == CODED)
      {
        if (keyCode == RIGHT)
          title.menuSelection = PApplet.parseByte((title.menuSelection + 1)%3);
        if (keyCode == LEFT) 
          if (title.menuSelection == 0)
            title.menuSelection = 2;
          else
            title.menuSelection = PApplet.parseByte((title.menuSelection - 1)%3);
      }
      break;
      
    case (GS_HELP):
      // to exit help early
      if (key == ENTER || key == RETURN || key == ESC)  
        help.reset = true;
      // to speed up or slow down help text
      if (key == CODED)
      {
        if (keyCode == UP)
          help.speedY += 0.04f;
        if (keyCode == DOWN)
          help.speedY -= 0.04f;
      }
      break;
      
    case (GS_SCORES):
      if (key == ENTER || key == RETURN)
        gameState = GS_TITLE;
      break;
      
    case (GS_GAME_OVER):
      if (key == ENTER || key == RETURN || key == ESC) 
      {
        loop(); 
        gameState = GS_TITLE;
      }
      break;
      
    case (GS_PLAYING):
      if (key == CODED){
        if (keyCode == RIGHT){
          rightHeld = true;
        }
        if (keyCode == LEFT){
          leftHeld = true;
        }
      }
      break;
  }  // End switch (gameState)
}

public void keyReleased()
{
  if (key == CODED){
    if (keyCode == RIGHT){
      rightHeld = false;
    }
    if (keyCode == LEFT){
      leftHeld = false;
    }
  }
} 
class Blob {
  public static final float SPAWN_TIME = 2000.0f;
  private PVector pos;
  float rad;
  public LinkedList bubbles;
  Circle bounding_circle;  // Circle for detecting collisions
  private int[] hit_time = new int[2]; //blob can get hit twice
  private PVector[] hit_pos = new PVector[2];
  
  Blob(int x, float r){
    pos = new PVector(x, screeny);
    rad = r;
    hit_time[0] = 0;
    hit_time[1] = 0;
    bubbles = new LinkedList();
//    bubbles.add(new Bubble(new PVector(pos.x+.8*rad, pos.y-.8*rad)));
    bounding_circle = new Circle(pos, rad);
  }
  
  //return true if blob was destroyed, otherwise false
  public boolean collide(Comet c)
  {
    int temp_hit_time = millis();

    //check each bubble for collision with the given comet
    for(int i=0; i < bubbles.size(); i++)
    {
      Bubble b = (Bubble) bubbles.get(i);

      boolean destroyed = b.collide(c);
      if(destroyed)
      {
        bubbles.remove(b);
      }
      else
      {
        Bubble new_bubble = b.regen();
        if(new_bubble != null)
        {
          bubbles.add(new_bubble);
        }
      }
    }
    if (c.mark_for_deletion) return false;
    
    //check if the comet hit the main blob
    PVector collision_pos = bounding_circle.collide(c.bcircle);
    if (collision_pos != null){ // Collision with main blob
      c.mark_for_deletion = true;
      collision_pos.sub(bounding_circle.pos);
    
      if(hit_time[0] > 0 && hit_time[1] > 0) //third hit, so die
        return true;
        
      //Play comet hit sound
      if (cometHit.position() == (cometHit.length()-1)){
        cometHit.rewind();
      }
      cometHit.play();

      int index; //0 or 1 based on where hit stuff goes
      if (hit_time[0] == 0)
        index = 0;
      else index = 1;
      hit_time[index] = temp_hit_time;
      hit_pos[index] = PVector.sub(c.bcircle.pos, pos);
    }
    
    //didn't die, so check for regeneration
    PVector child_pos = null;
    for (int i=0; i<2; i++){
      if (hit_time[i] > 0 && hit_time[i] + SPAWN_TIME < temp_hit_time){
        child_pos = hit_pos[i];
        child_pos.add(pos);
        hit_time[i] = 0;
        hit_pos[i] = null;
      }
    }
    if (child_pos != null)
    {
      //play sound
       if (regen.position() >= regen.length()){
        regen.rewind();
       }
       regen.play();
      bubbles.add(new Bubble (child_pos));
    }
    return false;
  }

  public void update()
  {
    if(rightHeld && pos.x+rad/2 < screenx){
      pos.x += SPEED;
    }
    if(leftHeld && pos.x-rad/2 > 0){
      pos.x -= SPEED;
    }

    for(int i=0; i<bubbles.size(); i++){
      Bubble b = (Bubble) bubbles.get(i);
      if(leftHeld && pos.x-rad/2 > 0)
        b.moveLeft();
      if(rightHeld && pos.x+rad/2 < screenx)
        b.moveRight();
    }
  }

  public void disp()
  {
    //figure out which image to use. Note there is no 'else' here.
    PImage img = blob_cleanimg;
    if(hit_time[0] + hit_time[1] > 0)
      img = blob_hurt1img; //one hole
    if(hit_time[0] + hit_time[1] > millis())
      img = blob_hurt2img; //two holes
      
//    ellipse(pos.x, pos.y, 2*rad, 2*rad);
    image(img, pos.x-rad, pos.y-rad, 2*rad, 2*rad);
    for(int i=0; i<bubbles.size(); i++){
      Bubble b = (Bubble) bubbles.get(i);
      b.disp();
    }
  }
}



class Bubble
{
  public static final float RADIUS = 10.0f;
  public static final float SPAWN_TIME = 2000.0f;
  public Circle bounding_circle;
  PVector pos;

  private int hit_state;
  private int hit_time; //hit_time = 0 means it's healthy
  private PVector hit_pos;
  
  public Bubble(PVector pos)
  {
    this.bounding_circle = new Circle(pos, RADIUS);
    this.pos = pos;
    hit_time  = 0;
  }
  
  public void moveRight()
  { pos.x += SPEED; }
  public void moveLeft()
  { pos.x -= SPEED; }

  public void disp()
  {
    if(hit_time != 0)
      image(bub_hurtimg, pos.x-RADIUS, pos.y-RADIUS, RADIUS, RADIUS);
    else image(bub_cleanimg, pos.x-RADIUS, pos.y-RADIUS, RADIUS, RADIUS);
//    if() stroke(255, 0, 0);
//    else stroke(0, 255, 0);
//    ellipse(pos.x, pos.y, RADIUS, RADIUS);
  }
  
  /**
  * Returns true if the bubble was killed, false otherwise
  */
  public boolean collide(Comet com)
  {
    Circle c = com.bcircle;
    PVector collision_pos = bounding_circle.collide(c);
    
    //No Collision
    if(collision_pos == null)
      return false;
      
    //Collision
    else
    {
      com.mark_for_deletion = true;
      //play comet hit sound
      if (cometHit.position() == (cometHit.length()-1)){
        cometHit.rewind();
      }
      cometHit.play();
      
      if (hit_time > 0)
        return true;
      
      hit_time = millis();
      hit_pos = PVector.sub(c.pos, pos);
      collision_pos.sub(bounding_circle.pos);
    }
    return false;
  }
  
  /**
  * Generates a new bubble as the child of this one and returns it
  * or returns null
  */
  public Bubble regen()
  {
    if(hit_time == 0) return null;
    if(millis() - hit_time < SPAWN_TIME) return null;

    //Grow new bubble from damaged quadrant
    hit_pos.mult(.8f);
    hit_pos.add(pos);
    PVector child_pos = hit_pos;
      
    //Grow back quad
    hit_time = 0;
    hit_pos = null;
    //play sound
    if (regen.position() >= regen.length()){
      regen.rewind();
    }
    regen.play();
    return new Bubble(child_pos);
  }
}
class Circle
{
  public PVector pos;
  public float rad;
  
  public Circle(PVector pos, float rad)
  {
    this.pos = pos;
    this.rad = rad;
  }
  
  /**
  * Returns point of collision with given circle
  * or null otherwise
  */
  public PVector collide(Circle c)
  {
    float collide_distance = this.rad + c.rad;
    float distance         = PVector.dist(this.pos, c.pos);
    
    if(distance < collide_distance)
    {
      //Collision
      PVector point_of_collision = new PVector(c.pos.x, c.pos.y);
      point_of_collision.sub(this.pos);
      point_of_collision.mult(this.rad / (this.rad + c.rad));
      point_of_collision.add(this.pos);
      
      return point_of_collision;
    }
    else
    {
      //No collision
      return null;
    }
  }
  
}
class Comet {
  float y;
  public Circle bcircle;
  public boolean mark_for_deletion;
  
  Comet() //create a Comet with random x
  {
    int x = PApplet.parseInt(random(screenx));
    y = 0;
    bcircle = new Circle(new PVector(x, y), 1);
  }
  
  public void update()
  {
    y = y + (20.0f+level)/10.0f; //gets faster every level
    bcircle.pos.y = y;
  }
  
  public void disp()
  {
//    line(bcircle.pos.x, y, bcircle.pos.x, y+5);
    image(comet_img, bcircle.pos.x-3, y-30, 6, 30);
  }
}
class HelpScreen
{
  Blob blob;
  Comet comet;
  Bubble bubble;
  float transY = 3*height/4, // Keeps how much to translate by Y
        speedY = .5f; // Rate at which transY changes/frame
  boolean reset = false;

  final String strings[] =
  { 
    "On playing Regen, and the joys thereof:",
    "This is you",
    "This is a comet",
    "Use the arrow keys to move side to side to avoid or intercept"
      + " these",
    "If you are hit by a comet, you will be wounded",
    "From here, one of two things will happen:",
    "If you are hit soon after two more times, you will perish",
    "Otherwise, from the hurt of the past shall spring the hope"
      + " of the future",
    "If your child is hit, it may have a child of its own",
    "However, if a child is hit before it can regenerate,"
      + " it shall perish",
    "Grow tall and wide, for such is your safety!"
  };
  
  final int textBoxWidth = width/2,     // max text section width
        textYSpacing = 70, // max text height
        textX = width/4;   // text x location
//  int textY = textYSpacing;   // text y location
  
  /** Constructor */
  HelpScreen()
  {
    blob = new Blob(width/2, BLOB_RADIUS);
    blob.pos.x = width - (5 + 2*blob.rad);
    
    comet = new Comet();
    comet.bcircle.pos.x = width - (5 + 2*blob.rad);
    
    // The y-coord in this constructor will be changed later
    bubble = new Bubble(new PVector(width - (5 + 2*blob.rad),0));
  }
  
  public void display()
  {
    if (!reset) {
      background(0);
 
      
      pushMatrix();
      translate(0, transY -= speedY);
      
      // draw demo blob
      blob.pos.y = 2 * textYSpacing;
      blob.disp();
      // draw demo comet
      comet.y = 3*textYSpacing;
      comet.disp();
      // draw demo hurt blob
      image(blob_hurt1img, 
            width - 3*blob.rad, 5*textYSpacing, 
            2*blob.rad, 2*blob.rad);
      // draw demo blob+bubble
      blob.pos.y = textYSpacing + blob.rad 
                  + 7*textYSpacing;
      blob.disp();
      bubble.pos.y = textYSpacing + 7*textYSpacing;
      bubble.disp(); 
      // draw demo bubble + child bubble
      bubble.pos.y = textYSpacing + 8.25f*textYSpacing;
      bubble.disp();
      bubble.pos.x += .7f * bubble.RADIUS;
      bubble.pos.y = textYSpacing - .7f*bubble.RADIUS 
                    + 8.25f*textYSpacing;
      bubble.disp();
      // restore bubble's x-coord
      bubble.pos.x -= .7f * bubble.RADIUS;
      
      // Print text
      for (int i = strings.length-1; i > 0; i--)
        text(strings[i], 
            textX, textYSpacing + i*textYSpacing, 
            textBoxWidth, textYSpacing);
            
      if (transY < -height)
        reset = true;
      
      popMatrix();
      
      // Title should be at the top
      fill(0);
      rect(0, 0, width, height/7);
      fill(255);
      text(strings[0], textX, textYSpacing);
    }
    else 
    {
      reset = false;
      transY = 3*height/4;
      gameState = GS_TITLE;
      speedY = 0.5f;
    }
  }
}
class ScoreScreen
{
  int scores[] = {0,0,0,0,0,0,0,0,0,0};  // Top 10 scores
  String scoresFromFile[];
  String line;        // Temp line read in from file
  String fileName = "Scores.rgn";  // Name of hi score file
  
  /** Constructor */
  ScoreScreen()
  {
    scoresFromFile = loadStrings(fileName);
    if (scoresFromFile != null)
      for (int i = 0; i < scoresFromFile.length && i < 10; i++)
        scores[i] = parseInt(scoresFromFile[i], 10);
  }
  
  /** Displays the high scores (if any)! */
  public void display()
  {
    if (scores[scores.length-1] == 0)
    {
      text("No Scores yet! Guess it's time to play!", 
            width/3, height/2);
      return;
    }
    
    text("Scores", width/2 - 10, 20);
    
    // display scores in descending order
    int number = 1;  // number of score
    int offsetX = 20, offsetY = 25;
    // Order is low to high, so start at end
    // (might be some empty slots (0) at beginning)
    for (int i = scores.length-1; i >= 0 && scores[i] > 0; i--)
      text("" + number++ + ".   " + scores[i],
            offsetX, offsetY*number);
    
    text("(Press enter to exit the Stage of History)",
          width/2-30, height-40);
  }
  
  /** Returns true if the given score is a high score */
  public boolean isHighScore(int score)
  {
    return score > scores[0];
  }
  
  /** Adds a score to the list */
  public void addScore(int score)
  {
    // lowest score will be at 
    // the first index
    scores[0] = score;
    Arrays.sort(scores);
  }
  
  /** Prints the scores to a file */
  public void saveScores()
  {
    String[] tmp = new String[10];
    for (int i = 0; i < scores.length; i++)
      tmp[i] = "" + scores[i];
    saveStrings(fileName, tmp);
  }
}

/** A class for displaying a title screen */
class TitleScreen {
  // Menu options
  static final byte ID_BEGIN  = 0;
  static final byte ID_LEARN  = 1;
  static final byte ID_SCORES = 2;
  
  // Bitmap variables
  static final byte BUBBLE = 1;
  static final byte COMET  = 2;
  
  // Spacing for bitmaps
  static final int BUBBLE_RADIUS = 5;
  static final int COMET_RADIUS  = 7;
  
  Blob arrowBlob;    // Selection arrow blob

  byte menuSelection = ID_BEGIN;  
  
  /* Bitmap displays made out of bubbles or comets -
     spacing is staggered to approximate bubble spacing.
     1's represent bubbles, 2's represent comets
   */
   // "Regen" in blobs
  final byte TITLE[][] =
  {  // R            e               g          e               n
    {0,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
     {0,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
     {0,1,0,0,0,1,0,0,1,1,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,0,0,1,0,1,1,1,0,0,0},
    {0,1,0,0,0,0,1,0,1,1,0,1,0,0,0,1,1,0,1,1,0,0,0,1,1,0,1,0,0,1,1,1,1,1,0,0},
     {0,1,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0,0,1,0,0,0,1,0,0,0,1,0,1,1,0,0,0,1,0,0},
    {0,1,0,0,1,1,0,1,1,1,1,1,1,0,1,0,0,0,0,1,0,0,1,1,1,1,1,1,0,1,0,0,0,0,1,0},
     {0,1,1,1,0,0,1,1,1,1,1,1,0,1,0,0,0,0,1,0,0,1,1,1,1,1,1,0,1,0,0,0,0,0,1,0},
    {0,1,0,1,0,0,1,0,0,0,0,0,0,1,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,0,0,0,1,0},
     {0,1,0,1,0,1,0,0,0,0,0,0,1,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,1,0,0,0,0,0,1,0},
    {0,1,0,0,1,0,1,0,0,0,0,1,0,1,0,0,0,0,1,1,0,1,0,0,0,0,1,0,0,1,0,0,0,0,1,0},
     {0,1,0,0,1,0,1,1,1,1,1,0,0,1,1,1,1,1,1,0,0,1,1,1,1,1,0,0,1,0,0,0,0,0,1,0},
    {0,1,1,0,0,1,0,1,1,1,1,0,0,0,1,1,1,1,0,1,0,0,1,1,1,1,0,0,1,1,0,0,0,0,1,1},
     {0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
     {0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}
  };
  
  // "Begin" in comets
  final byte BEGIN[][] = 
  {
    {2,2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0},
    {2,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {2,0,0,2,0,0,0,2,2,0,0,0,0,2,2,0,2,0,0,0,2,0,2,2,0,0},
    {2,2,2,2,0,0,2,0,0,2,0,0,2,0,2,0,2,0,0,0,2,2,0,0,2,0},
    {2,0,0,0,2,0,2,2,2,2,0,2,0,0,2,0,2,0,0,0,2,0,0,0,2,0},
    {2,0,0,0,2,0,2,0,0,0,0,2,0,0,2,0,2,0,0,0,2,0,0,0,2,0},
    {2,0,0,0,2,0,2,0,0,2,0,2,0,2,2,0,2,0,2,0,2,0,0,0,2,0},
    {2,2,2,2,0,0,0,2,2,0,0,0,2,0,2,0,0,2,0,0,2,0,0,0,2,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,2,0,0,2,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,2,2,0,0,0,0,0,0,0,0,0,0,0,0}
  };
  
  // "Learn" in comets
  final byte LEARN[][] = 
  { // L     e         a           r         n
    {2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {2,0,0,0,0,2,2,0,0,0,0,2,0,2,0,2,0,2,0,0,2,0,2,2,0,0},
    {2,0,0,0,2,0,0,2,0,0,2,0,2,2,0,2,2,0,2,0,2,2,0,0,2,0},
    {2,0,0,0,2,2,2,2,0,2,0,0,0,2,0,2,0,0,0,0,2,0,0,0,2,0},
    {2,0,0,0,2,0,0,0,0,2,0,0,0,2,0,2,0,0,0,0,2,0,0,0,2,0},
    {2,0,0,0,2,0,0,2,0,2,0,0,2,2,0,2,0,0,0,0,2,0,0,0,2,0},
    {2,2,2,0,0,2,2,0,0,0,2,2,0,2,0,2,0,0,0,0,2,0,0,0,2,0},
  };
  
  // "Scores" in comets
  final byte SCORES[][] =
  { //S        c       o       r         e         s
    {0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {2,0,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {2,0,0,0,0,0,2,0,0,0,2,0,0,2,0,2,0,0,0,2,2,0,0,0,2,0},
    {0,2,0,0,0,2,0,2,0,2,0,2,0,2,2,0,2,0,2,0,0,2,0,2,0,2},
    {0,0,2,0,0,2,0,0,0,2,0,2,0,2,0,0,2,0,2,2,2,2,0,0,2,0},
    {0,0,0,2,0,2,0,0,0,2,0,2,0,2,0,0,0,0,2,0,0,0,0,0,0,2},
    {2,0,0,2,0,2,0,2,0,2,0,2,0,2,0,0,0,0,2,0,0,2,0,2,0,2},
    {0,2,2,0,0,0,2,0,0,0,2,0,0,2,0,0,0,0,0,2,2,0,0,0,2,0}
  };
  
  // Bubble selection arrow
  final byte BUBBLE_ARROW[][] = 
  {
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
     {0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0},
     {0,0,0,0,0,1,1,1,0,1,1,1,0,0,0,0,0,0,0},
    {0,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0,0,0},
     {0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0},
     {0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0}
  };
  
  /** Constructor */
  public TitleScreen() 
  {
    arrowBlob = new Blob(width/4, BLOB_RADIUS);
  }
  
  
  /** Display the title screen.  */
  public void display() 
  {
    displayBitmap(0,10,TITLE,     // The title
                  TITLE[0].length, 
                  TITLE.length, 
                  BUBBLE_RADIUS);
    displayBitmap(5, height/3, BEGIN,  // "Begin"
                  BEGIN[0].length, 
                  BEGIN.length, 
                  COMET_RADIUS); 
    displayBitmap(width/3, height/2, LEARN,  // "Learn"
                  LEARN[0].length, 
                  LEARN.length, 
                  COMET_RADIUS); 
    displayBitmap(width/2, 2*height/3, SCORES,  // "Scores"
                  SCORES[0].length, 
                  SCORES.length, 
                  COMET_RADIUS);
    
    //////////// display arrow blob /////////////
    // First, check what menu item is selected
    // and move to that item.
    switch (menuSelection)
    {
      case ID_BEGIN:
        arrowBlob.pos.x = width/10;
        break;
      case ID_LEARN:
        arrowBlob.pos.x = width/3;
        break;
      case ID_SCORES:
        arrowBlob.pos.x = width/2;
    }
    // Then, draw the blob and arrow
    arrowBlob.disp();
    
    displayBitmap(PApplet.parseInt(arrowBlob.pos.x - (BUBBLE_ARROW.length-1)*BUBBLE_RADIUS*2), 
                  PApplet.parseInt(height-PApplet.parseInt(BLOB_RADIUS)-BUBBLE_ARROW[0].length*(BUBBLE_RADIUS-.8f)), 
                  BUBBLE_ARROW, 
                  BUBBLE_ARROW[0].length, 
                  BUBBLE_ARROW.length, 
                  BUBBLE_RADIUS);
  }
  
  /** Display a bitmap
      @param x  The upper-left x coordinate for the bitmap
      @param y  The upper-left y coordinate for the bitmap
      @param bitmap  The array from which to draw a bitmap
      @param wide  The width of the bitmap, in elements
      @param high  The height of the bitmap, in elements
      @param radius   The radius that the elements should have
  */
  public void displayBitmap(int x, int y, byte bitmap[][], int wide, int high, int radius)
  {
    pushMatrix();
    translate(x,y);
    
    int offset = 0;  // Amount to offset rows when drawing
    for (int i = 0; i < high; i++)
      for (int j = 0; j < wide; j++)
        switch (bitmap[i][j])
        {
          case 1:  // Bubble
            if (i % 2 == 0)  // offset every other row
              offset = 0;
            else 
              offset = radius;
            new Bubble(new PVector(j*radius*2 + offset, i*radius*2)).disp();
            break;
          case 2:  // Comet
            Comet comet = new Comet();
            // Comet's x and y must be set manually.
            comet.bcircle.pos.x = j*radius;
            comet.y             = i*radius;
            comet.disp();
            break;
        }  // End switch(bitmap[i][j]
    
    popMatrix();
  }
}



  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#F0F0F0", "Regen" });
  }
}
