import ddf.minim.*;

int screenx = 400;
int screeny = 500;
final int SPEED = 2;
int level = 1;
Blob blob;
LinkedList comets;
ScoreScreen score;
int counter; //counts score
boolean rightHeld = false;
boolean leftHeld = false;
int startTime = 0;  // play start time in millis
final int BLOB_X = 100;
final float BLOB_RADIUS = 20.0;

//images from data file
PImage bg;
PImage blob_cleanimg;
PImage blob_hurt1img;
PImage blob_hurt2img;
PImage bub_cleanimg;
PImage bub_hurtimg;
PImage comet_img;
  
// Game States
final byte GS_PLAYING = 0;  // Playing game
final byte GS_HELP = 1;  // At help/learn screen
final byte GS_SCORES = 2;  // At High Score screen
final byte GS_TITLE = 3;  // At title screen
final byte GS_GAME_OVER = 4;  // At game over screen
byte gameState = GS_TITLE;

//sound
AudioSnippet bkgmusic;
AudioSnippet cometHit;
AudioSnippet regen;
AudioSnippet gameover;
AudioSnippet helpmusic;
Minim minim;

TitleScreen title;
HelpScreen  help;

void setup() 
{
  size(screenx, screeny);
  
  loadImages();
  loadAudio();

  blob = new Blob(BLOB_X, BLOB_RADIUS);
  comets = new LinkedList();
  
  title = new TitleScreen();
  help  = new HelpScreen();
  score = new ScoreScreen();
}

void loadAudio()
{
  minim = new Minim(this);
  bkgmusic = minim.loadSnippet("spaceloop2.wav");
  cometHit = minim.loadSnippet("comet_short.mp3");
  regen = minim.loadSnippet("effectgame.wav");
  gameover = minim.loadSnippet("comet_long.mp3");
  //actually found this annoying, but it's here if we decide to use it
  helpmusic = minim.loadSnippet("spaceloop1.mp3");
}

void loadImages()
{
  bg = loadImage("bg.jpg");
  blob_cleanimg = loadImage("bubble6.png");
  blob_hurt1img = loadImage("bubble6_hurt.png");
  blob_hurt2img = loadImage("bubbledamagedcenter.png");
  bub_cleanimg = loadImage("extrabubble2.png");
  bub_hurtimg = loadImage("extrabubbledmg.png");
  comet_img = loadImage("cometRed.png");
}

//return the probability that a new comet is made
//at the current timestep, given the level
float newCometProb()
{
  return (2.0 + 3.0*level) / (60.0 + 3.0*level);
}

//create a faller
void makeComet()
{
  comets.add(new Comet());
}

void die()
{
  //halt any sound effects
  bkgmusic.pause();
  cometHit.pause();
  regen.pause();
  background(bg);
  
  gameState = GS_GAME_OVER;
  gameover.play();
  text("You died :(", 160, 170);
  text("level: "+level, screenx-60, 14); //draw the level
  text("comets: "+comets.size(), 5, 14);
  text("bubbles: "+blob.bubbles.size(), 5, 28);
  text("max bubbles: "+counter, 5, 42);
  
  noLoop();
  gameover.rewind();
  // Compute score and save it if it's high
  int tmpScore = (level + 1) * (counter);
  counter = 0;
  if (score.isHighScore(tmpScore))
  {
    text("You got a high score of " + tmpScore + '!',
          150, 190);
    score.addScore(tmpScore);
    // Save score everytime there's an update,
    // in case of a sudden exit
    score.saveScores();
  }
  
  // reset everything
  startTime = 0;
  level = 0;
  blob = new Blob(BLOB_X, BLOB_RADIUS);
  comets = new LinkedList();
}

void draw()
{
  if (bg != null)
    background(bg);
  else background(0);
  
  switch (gameState)
  {
    case (GS_TITLE):
      title.display();    // Display the title screen
      break;
      
    case (GS_HELP):       // Display the help screen
      help.display();
      break;
      
    case (GS_SCORES):     //Display the score screen
      score.display();
      break;
      
    case (GS_PLAYING):
      if (startTime == 0)     // set start to zero
        startTime = millis(); // when a new game starts
      level = (millis()-startTime)/10000;
      //sound
      if (bkgmusic.position() >= (bkgmusic.length()-1))
      {
        bkgmusic.rewind();
      }
      bkgmusic.play();
      if (random(1) < newCometProb())
        makeComet(); //create a faller
      for(int i=0; i<comets.size(); i++){
        Comet c = (Comet) comets.get(i);
        c.update();
        if(blob.collide(c)){
          die();
          return;
        }
        if(c.bcircle.pos.y > screeny || c.mark_for_deletion)
          comets.remove(c);
        stroke(0, 0, 255);
        c.disp();    
      }
      blob.update();
      blob.disp();
      
      if (blob.bubbles.size() > counter)
        counter = blob.bubbles.size();
    
      text("level: "+level, screenx-60, 14); //draw the level
      text("comets: "+comets.size(), 5, 14);
      text("bubbles: "+blob.bubbles.size(), 5, 28);
      text("max bubbles: "+counter, 5, 42);
      break;
  }  // End switch(gameState)
}

void keyPressed()
{
  // Check which screen we're on
  switch (gameState)
  {
    case (GS_TITLE):
      if (key == ENTER || key == RETURN)
        switch (title.menuSelection)
        {
          case (TitleScreen.ID_LEARN):
            gameState = GS_HELP;
            break;
          case (TitleScreen.ID_BEGIN):
            gameState = GS_PLAYING;
            break;
          case (TitleScreen.ID_SCORES):
            gameState = GS_SCORES;
            break;
        }  // End switch(title.menuSelection)
      if (key == CODED)
      {
        if (keyCode == RIGHT)
          title.menuSelection = byte((title.menuSelection + 1)%3);
        if (keyCode == LEFT) 
          if (title.menuSelection == 0)
            title.menuSelection = 2;
          else
            title.menuSelection = byte((title.menuSelection - 1)%3);
      }
      break;
      
    case (GS_HELP):
      // to exit help early
      if (key == ENTER || key == RETURN || key == ESC)  
        help.reset = true;
      // to speed up or slow down help text
      if (key == CODED)
      {
        if (keyCode == UP)
          help.speedY += 0.04;
        if (keyCode == DOWN)
          help.speedY -= 0.04;
      }
      break;
      
    case (GS_SCORES):
      if (key == ENTER || key == RETURN)
        gameState = GS_TITLE;
      break;
      
    case (GS_GAME_OVER):
      if (key == ENTER || key == RETURN || key == ESC) 
      {
        loop(); 
        gameState = GS_TITLE;
      }
      break;
      
    case (GS_PLAYING):
      if (key == CODED){
        if (keyCode == RIGHT){
          rightHeld = true;
        }
        if (keyCode == LEFT){
          leftHeld = true;
        }
      }
      break;
  }  // End switch (gameState)
}

void keyReleased()
{
  if (key == CODED){
    if (keyCode == RIGHT){
      rightHeld = false;
    }
    if (keyCode == LEFT){
      leftHeld = false;
    }
  }
} 
